package ch.inf3.indexation.server;

import ch.inf3.indexation.crawler.PageCrawler;
import ch.inf3.indexation.updater.UpdateTaskRunnable;
import java.util.concurrent.Future;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.enterprise.concurrent.ManagedExecutorService;
import javax.enterprise.concurrent.ManagedScheduledExecutorService;
import javax.inject.Inject;
import javax.inject.Singleton;
import javax.persistence.EntityManager;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import org.apache.solr.client.solrj.impl.HttpSolrClient;
import org.eclipse.microprofile.faulttolerance.ExecutionContext;
import org.eclipse.microprofile.faulttolerance.Fallback;
import org.eclipse.microprofile.faulttolerance.FallbackHandler;
import org.eclipse.microprofile.faulttolerance.Timeout;

@Path("/task")
@Singleton
public class TaskController {

    private static final Logger LOG = Logger.getLogger(TaskController.class.getName());

    @Resource
    private ManagedScheduledExecutorService exec;

    @Resource
    private ManagedExecutorService executorService;

    @Inject
    private EntityManager em;

    @Inject
    private HttpSolrClient solr;

    private ScheduledFuture<?> updateTask;
    private UpdateTaskRunnable crawler;

    @PostConstruct
    private void registerTask() {
        LOG.log(Level.FINER, "Registering task for crawling updates");

        crawler = new UpdateTaskRunnable(executorService, em, solr);
        updateTask = exec.scheduleAtFixedRate(crawler, 0, 30, TimeUnit.MINUTES);
    }

    @POST
    @Path("/{bookmarkId}")
    @Timeout(500)
    @Fallback(TimeoutFallbackHandler.class)
    @Produces(MediaType.APPLICATION_JSON)
    public Response addSingleTask(@PathParam("bookmarkId") String bookmarkId) {
        PageCrawler pageCrawler = new PageCrawler(em, bookmarkId, solr, null);
        Future<Boolean> success = executorService.submit(pageCrawler);

        return Response
                .status(Response.Status.ACCEPTED)
                .build();
    }

    @GET
    @Produces(MediaType.APPLICATION_JSON)
    public Response listTasks() {
        long delay = updateTask.getDelay(TimeUnit.SECONDS);

        return Response
                .ok(new TaskListJson(delay))
                .build();
    }

    @DELETE
    public Response cancelTask() {
        boolean c = updateTask.cancel(true);

        return c ? Response.ok().build() : Response.serverError().build();
    }

    public static class TaskListJson {

        private final Long delay;

        public TaskListJson(Long delay) {
            this.delay = delay;
        }

        public Long getDelay() {
            return delay;
        }
    }

    public static class TimeoutFallbackHandler implements FallbackHandler<Response> {

        @Override
        public Response handle(ExecutionContext ec) {
            return Response
                    .status(Response.Status.SERVICE_UNAVAILABLE)
                    .build();
        }

    }
}
